using System;
using System.Collections.Generic;
using DXSK8.Service;
using DevExpress.Data.Filtering;
using DevExpress.ExpressApp;
using DevExpress.ExpressApp.Updating;

namespace DXSK8.Admin.Module.DatabaseUpdate {
    public class Updater : ModuleUpdater {
        public Updater(IObjectSpace objectSpace, Version currentDBVersion)
            : base(objectSpace, currentDBVersion) {
        }

        public override void UpdateDatabaseAfterUpdateSchema() {
            base.UpdateDatabaseAfterUpdateSchema();
            if (ObjectSpace.FindObject<Employee>(CriteriaOperator.Parse("UserName=?","Admin")) == null) {
                var roleAndUser = CreateRoleAndUser("Admin");
                roleAndUser.IsAdministrative = true;
                CreateRole("Content Manager", new[]{
                    CRUDPermission(typeof(Product)),
                    ReadOnlyPermission(typeof(Order)),
                    ReadOnlyPermission(typeof(Employee))
                });

                CreateRole("Sales Manager", new[]{
                    CRUDPermission(typeof(Order)),
                    ReadOnlyPermission(typeof(Product)),
                    ReadOnlyPermission(typeof(Employee))
                });

                var employees = ObjectSpace.GetObjects<Employee>();
                foreach (var employee in employees) {
                    employee.IsActive = true;
                    employee.ChangePasswordOnFirstLogon = true;
                }

                ObjectSpace.CommitChanges();
            }
        }

        TypePermissionObject ReadOnlyPermission(Type type) {
            return CreateTypePermissionObject(type, false, false, true, true, false);
        }

        TypePermissionObject CRUDPermission(Type type) {
            return CreateTypePermissionObject(type, true, true, true, true, true);
        }

        TypePermissionObject CreateTypePermissionObject(Type targetType, bool allowCreate, bool allowDelete, bool allowNavigate,
                                                        bool allowRead, bool allowWrite) {
            var typePermissionObject = ObjectSpace.CreateObject<TypePermissionObject>();
            typePermissionObject.TargetType = targetType;
            typePermissionObject.AllowCreate = allowCreate;
            typePermissionObject.AllowDelete = allowDelete;
            typePermissionObject.AllowNavigate = allowNavigate;
            typePermissionObject.AllowRead = allowRead;
            typePermissionObject.AllowWrite = allowWrite;
            return typePermissionObject;
        }

        Role CreateRoleAndUser(string name) {
            var role = CreateRole(name);
            var employee = ObjectSpace.CreateObject<Employee>();
            employee.IsActive = true;
            employee.UserName = name;
            employee.Position=name;
            employee.Person = Person.CreatePerson(name, name, name);
            employee.Avatar = ObjectSpace.CreateObject<ImageSource>();
            employee.Avatar.Image=new byte[0];
            employee.Roles.Add(role);
            return role;
        }

        Role CreateRole(string name) {
            var role = ObjectSpace.CreateObject<Role>();
            role.Name = name;
            return role;
        }


        void CreateRole(string name, IEnumerable<TypePermissionObject> permissionObjects) {
            var role = CreateRole(name);
            foreach (var typePermissionObject in permissionObjects) {
                role.TypePermissions.Add(typePermissionObject);
            }
        }
    }
}