using System;
using System.Collections.Generic;
using DevExpress.ExpressApp;
using DevExpress.ExpressApp.DC;
using DevExpress.ExpressApp.EF;
using DevExpress.ExpressApp.Updating;
using Updater = DXSK8.Admin.Module.DatabaseUpdate.Updater;


namespace DXSK8.Admin.Module {
    public sealed partial class AdminModule : ModuleBase {
        public AdminModule() {

            InitializeComponent();

            AdditionalExportedTypes.Add(typeof(DXSK8.Service.Customer));
            AdditionalExportedTypes.Add(typeof(DXSK8.Service.Address));
            AdditionalExportedTypes.Add(typeof(DXSK8.Service.Brand));
            AdditionalExportedTypes.Add(typeof(DXSK8.Service.Employee));
            AdditionalExportedTypes.Add(typeof(DXSK8.Service.ImageSource));
            AdditionalExportedTypes.Add(typeof(DXSK8.Service.Login));
            AdditionalExportedTypes.Add(typeof(DXSK8.Service.NotificationChannel));
            AdditionalExportedTypes.Add(typeof(DXSK8.Service.Order));
            AdditionalExportedTypes.Add(typeof(DXSK8.Service.OrderItem));
            AdditionalExportedTypes.Add(typeof(DXSK8.Service.Person));
            AdditionalExportedTypes.Add(typeof(DXSK8.Service.Product));
            AdditionalExportedTypes.Add(typeof(DXSK8.Service.ProductFlavor));
            AdditionalExportedTypes.Add(typeof(DXSK8.Service.ProductType));
            AdditionalExportedTypes.Add(typeof(DXSK8.Service.Store));
        }

        public override void Setup(XafApplication application) {
            base.Setup(application);
            application.CustomCheckCompatibility+=ApplicationOnCustomCheckCompatibility;
            application.CreateCustomObjectSpaceProvider += ApplicationOnCreateCustomObjectSpaceProvider;
        }

        void ApplicationOnCustomCheckCompatibility(object sender, CustomCheckCompatibilityEventArgs customCheckCompatibilityEventArgs) {
            var objectSpace = customCheckCompatibilityEventArgs.ObjectSpaceProvider.CreateUpdatingObjectSpace(true);
            var updater = new Updater(objectSpace, null);
            updater.UpdateDatabaseAfterUpdateSchema();
        }


        void ApplicationOnCreateCustomObjectSpaceProvider(object sender, CreateCustomObjectSpaceProviderEventArgs e) {
            var typesInfo = (TypesInfo)Application.TypesInfo;
            const string metadata = "res://*/SK8Data.csdl|res://*/SK8Data.ssdl|res://*/SK8Data.msl";
            const string provider = "System.Data.SqlClient";
            var objectContextType = typeof(DXSK8.Service.SK8DataContainer);
            e.ObjectSpaceProviders.Add(new EFObjectSpaceProvider(objectContextType, typesInfo, null, e.ConnectionString, metadata, provider));
        }



        public override IEnumerable<ModuleUpdater> GetModuleUpdaters(IObjectSpace objectSpace, Version versionFromDB) {
            ModuleUpdater updater = new DatabaseUpdate.Updater(objectSpace, versionFromDB);
            return new[] { updater };
        }
    }
}
